#ifndef MUSICLIBRARY_H
#define MUSICLIBRARY_H

#include <QObject>
#include "musiclibrary_global.h"
#include "artist.h"
#include "record.h"
#include "song.h"

/**
 * Root class of a music library
 */
class MUSICLIBRARYSHARED_EXPORT MusicLibrary : public QObject
{
    Q_OBJECT

public:

    /**
     * Creates a new music library
     */
    MusicLibrary(QObject *parent = 0);

    /**
     * Deletes music library and all artists, records and songs it owns
     */
    ~MusicLibrary();

    /**
     * Gets all artists as list
     */
    QList<Artist *> artists() const;

    /**
     * Gets all records as list
     */
    QList<Record *> records() const;

    /**
     * Gets all songs as list
     */
    QList<Song *> songs() const;

signals:

    /**
     * Emitted when one of the library objects is changed
     */
    void objectChanged(MusicObject *obj);

};

#endif // MUSICLIBRARY_H
